#include <iostream>
#include <iomanip>
#include <cstdlib> // near equivalent to <stdlib.h>  for exit, rand, ...
#include <cmath>   // near equivalent to <math.h>    for sin, cos, log, ..

#include "mm.hh"
#include "TimeMeter.hh"


using namespace std ;

//
//  Fill with random value matric A (n x m)
//
static // routine visible only in "this file"
void
fillRandom( double A[], unsigned ldA, unsigned n, unsigned m ) {
  #define A(I,J) A[(I)+(J)*ldA]
  for ( unsigned i = 0 ; i < n ; ++i )
    for ( unsigned j = 0 ; j < m ; ++j )
      A(i,j) = rand()-rand() ; // IS NOT 0!
  #undef A
}

#define NREPEAT 2
#define n 640

int
main() {

  // for timing
  long from_sec, from_usec ;
  long to_sec, to_usec ;
  long sec, usec ;

  // declare and allocate matrices A, B and C
  static double A[n*n] ;
  static double B[n*n] ;
  static double C[n*n] ;

  // fill matrices A, B with random numbers
  fillRandom( A, n, n, n ) ;
  fillRandom( B, n, n, n ) ;

  getTime( from_sec, from_usec ) ;
  for ( int kk = 0 ; kk < NREPEAT ; ++kk ) {
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
    mm_standard( A, n, B, n, C, n, n, n, n ) ;
  }
  getTime( to_sec, to_usec ) ;

  timeDifference( from_sec, from_usec, to_sec, to_usec, sec, usec ) ;

  double elapsed = sec+1E-6*usec ;
  
  cout << "Elapsed time = " << elapsed/NREPEAT/10 << " seconds\n" ;

  return 0 ;
}
